#include "memory_incremental.h"
#include "pipe_opener_resource.h"
#include "pipe_opener_server.h"
#include "pipe_pager.h"
#include "resource_server.h"



/* Support for providing access to pipes. */

PipeOpenerResource::PipeOpenerResource(Memory *memory)
: _memory(memory)
{
}

int PipeOpenerResource::expected_items()
{
    return PipeOpener_expected_items;
}

ipc_server_handler_type PipeOpenerResource::handler()
{
    return (ipc_server_handler_type) handle_PipeOpener;
}



/* Pipe opener interface methods. */

long PipeOpenerResource::pipe(offset_t size, l4_cap_idx_t *reader, l4_cap_idx_t *writer)
{
    /* Both endpoints will employ a common paging coordinator. */

    PipePaging *paging = new PipePaging(_memory, size);
 
    /* Each endpoint will have its own pager. */

    /* NOTE: Failure to open an endpoint should invalidate both, plus the
             paging object. Also, any active server thread would need to be
             cancelled. */

    return open_endpoint(paging, false, reader) || open_endpoint(paging, true, writer);
}

long PipeOpenerResource::open_endpoint(PipePaging *paging, bool writing, l4_cap_idx_t *endpoint)
{
    PipePager *pager = new PipePager(paging, writing);

    /* Start the endpoint server in a new thread.
       If the thread does not start, the resource should be finalised. */

    ResourceServer server(pager);
    long err = server.start_thread();

    /* Return the server capability to the caller. */

    if (!err)
        *endpoint = server.config()->server;

    return err;
}

// vim: tabstop=4 expandtab shiftwidth=4
